# Spring Boot `JpaRepository` 教程

> 原文： [http://zetcode.com/springboot/jparepository/](http://zetcode.com/springboot/jparepository/)

SpringBoot `JpaRepository`教程展示了如何使用`JpaRepository`在 Spring Boot 应用中管理数据。

Spring 是流行的 Java 应用框架。 Spring Boot 致力于以最小的努力创建独立的，基于生产级别的基于 Spring 的应用。

## Spring Data

Spring Data 是用于数据访问的基于 Spring 的编程模型。 它减少了使用数据库和数据存储所需的代码量。 它由几个模块组成。 Spring Data JPA 简化了使用 JPA 技术的 Spring 应用的开发。

使用 Spring Data，我们为应用中的每个域实体定义了一个存储库接口。 存储库包含用于执行 CRUD 操作，对数据进行排序和分页的方法。 `@Repository`是标记注解，指示基础接口是存储库。 通过扩展特定的存储库接口（例如`CrudRepository`，`PagingAndSortingRepository`或`JpaRepository`）来创建存储库。

Spring Data 已与 Spring MVC 控制器进行了高级集成，并提供了从存储库方法名称派生的动态查询。

## `JpaRepository`

`JpaRepository`是`Repository`的 JPA 特定扩展。 它包含`CrudRepository`和`PagingAndSortingRepository`的完整 API。 因此，它包含用于基本 CRUD 操作的 API，以及用于分页和排序的 API。

## Spring Boot `JpaRepository` 示例

以下 Spring Boot 应用使用`JpaRepository`管理`City`实体。 数据保存在 H2 数据库中。 该应用是一个控制台程序。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           │   MyRunner.java
│   │           ├───model
│   │           │       City.java
│   │           └───repository
│   │                   CityRepository.java
│   └───resources
│           application.properties
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootjparepository</artifactId>
    <version>1.0-SNAPSHOT</version>

    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>

        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <scope>runtime</scope>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

这是 Maven 构建文件。 `spring-boot-starter-data-jpa`是将 Spring Data JPA 与 Hibernate 结合使用的入门工具。

`resources/application.properties`

```java
spring.main.banner-mode=off
logging.pattern.console=%clr(%d{yy-MM-dd E HH:mm:ss.SSS}){blue} %clr(%-5p) %clr(%logger{0}){blue} %clr(%m){faint}%n

```

`application.properties`是主要的 Spring Boot 配置文件。 使用`spring.main.banner-mode`属性，我们可以关闭 Spring 横幅。 `logging.pattern.console`定义控制台的日志模式。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "cities")
public class City {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(String name, int population) {

        this.name = name;
        this.population = population;
    }

    public Long getId() {

        return id;
    }

    public void setId(Long id) {

        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {

        this.name = name;
    }

    public int getPopulation() {

        return population;
    }

    public void setPopulation(int population) {

        this.population = population;
    }

    @Override
    public int hashCode() {

        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.id);
        hash = 79 * hash + Objects.hashCode(this.name);
        hash = 79 * hash + this.population;

        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }

        final City other = (City) obj;
        if (this.population != other.population) {
            return false;
        }

        if (!Objects.equals(this.name, other.name)) {
            return false;
        }

        return Objects.equals(this.id, other.id);
    }

    @Override
    public String toString() {

        var builder = new StringBuilder();
        builder.append("City{id=").append(id).append(", name=")
                .append(name).append(", population=")
                .append(population).append("}");

        return builder.toString();
    }
}

```

这是`City`实体。 它包含以下属性：`id`，`name`和`population`。

`com/zetcode/repository/CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.stereotype.Repository;

@Repository
public interface CityRepository extends JpaRepository<City, Long> {

}

```

`CityRepository`从`JpaRepository`延伸。 它提供了实体的类型及其主键。

> **注意**：在Java企业应用程序中，定义与存储库一起使用的服务层是一个好习惯。 为简单起见，我们跳过服务层。

`com/zetcode/MyRunner.java`

```java
package com.zetcode;

import com.zetcode.model.City;
import com.zetcode.repository.CityRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;

@Component
public class MyRunner implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(MyRunner.class);

    @Autowired
    private CityRepository cityRepository;

    @Override
    public void run(String... args) throws Exception {

        cityRepository.save(new City("Bratislava", 432000));
        cityRepository.save(new City("Budapest", 1759000));
        cityRepository.save(new City("Prague", 1280000));
        cityRepository.save(new City("Warsaw", 1748000));
        cityRepository.save(new City("Los Angeles", 3971000));
        cityRepository.save(new City("New York", 8550000));
        cityRepository.save(new City("Edinburgh", 464000));

        logger.info("# of cities: {}", cityRepository.count());

        logger.info("All cities unsorted:");
        var cities = cityRepository.findAll();
        logger.info("{}", cities);

        logger.info("------------------------");

        logger.info("All cities sorted by name in descending order");
        var sortedCities = cityRepository.findAll(new Sort(Sort.Direction.DESC, "name"));
        logger.info("{}", sortedCities);

        logger.info("------------------------");

        logger.info("Deleting all cities");
        cityRepository.deleteAllInBatch();

        logger.info("# of cities: {}", cityRepository.count());
    }
}

```

在`MyRunner`中，我们使用`JpaRepository`的各种方法。

```java
@Autowired
private CityRepository cityRepository;

```

我们将`CityRepository`注入`cityRepository`字段。

```java
cityRepository.save(new City("Bratislava", 432000));

```

使用`save()`插入一个新城市。

```java
logger.info("# of cities: {}", cityRepository.count());

```

我们用`count()`计算城市数。

```java
logger.info("All cities unsorted:");
var cities = cityRepository.findAll();
logger.info("{}", cities);

```

使用`findAll()`，我们可以到达所有城市。

```java
logger.info("All cities sorted by name in descending order");
var sortedCities = cityRepository.findAll(new Sort(Sort.Direction.DESC, "name"));
logger.info("{}", sortedCities);

```

通过将`Sort`对象传递给`findAll()`方法，我们得到了所有城市的降序排列。

```java
logger.info("Deleting all cities");
cityRepository.deleteAllInBatch();

```

我们使用`deleteAllInBatch()`批量删除所有城市。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`设置 Spring Boot 应用。

```java
...
19-06-25 Tue 12:47:14.593 INFO  MyRunner # of cities: 7
19-06-25 Tue 12:47:14.593 INFO  MyRunner All cities unsorted:
19-06-25 Tue 12:47:14.652 INFO  MyRunner [City{id=1, name=Bratislava, population=432000}, City{id=2, name=Budapest, population=1759000}, City{id=3, name=Prague, population=1280000}, City{id=4, name=Warsaw, population=1748000}, City{id=5, name=Los Angeles, population=3971000}, City{id=6, name=New York, population=8550000}, City{id=7, name=Edinburgh, population=464000}]
19-06-25 Tue 12:47:14.652 INFO  MyRunner ------------------------
19-06-25 Tue 12:47:14.652 INFO  MyRunner All cities sorted by name in descending order
19-06-25 Tue 12:47:14.667 INFO  MyRunner [City{id=4, name=Warsaw, population=1748000}, City{id=3, name=Prague, population=1280000}, City{id=6, name=New York, population=8550000}, City{id=5, name=Los Angeles, population=3971000}, City{id=7, name=Edinburgh, population=464000}, City{id=2, name=Budapest, population=1759000}, City{id=1, name=Bratislava, population=432000}]
19-06-25 Tue 12:47:14.668 INFO  MyRunner ------------------------
19-06-25 Tue 12:47:14.668 INFO  MyRunner Deleting all cities
19-06-25 Tue 12:47:14.681 INFO  MyRunner # of cities: 0
...

```

这是一个示例输出。

在本教程中，我们使用`JpaRepository`管理了应用数据。 您可能也对相关教程感兴趣： [Spring Boot `CrudRepository`教程](/springboot/crudrepository/)， [Spring Boot Data JPA `@Query`教程](/springboot/datajpaquery/)， [Spring Boot Data JPA `@NamedQuery`教程](/springboot/datajpanamedquery/)，[Java 教程](/lang/java/)。

列出[所有 Spring Boot 教程](/all/#springboot)。