# Spring Boot `Environment`

> 原文： [http://zetcode.com/articles/springbootenvironment/](http://zetcode.com/articles/springbootenvironment/)

Spring Boot `Environment`显示了如何在 Spring Boot 中读取环境变量。 Spring Boot 应用可以部署在各种环境中，在这种情况下读取环境变量可能会有所帮助。

Spring 是一种流行的 Java 应用框架，而 Spring Boot 是 Spring 的下一步发展，它可以帮助您以最少的精力创建独立的，生产级的基于 Spring 的应用。

`Environment`是代表当前应用运行环境的接口。 它可用于获取应用环境的配置文件和属性。

```java
$ echo %JAVA_HOME%
C:\Users\Jano\AppData\Local\Programs\Java\openjdk-11\

```

在此示例中，我们定义了`JAVA_HOME`环境变量。

```java
pom.xml
src
├── main
│   ├── java
│   │   └── com
│   │       └── zetcode
│   │           └── Application.java
│   └── resources
│       ├── application.properties
│       └── logback.xml 
└── test
    └── java

```

这是 Spring Boot 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootenvironment</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

这是 Maven 构建文件。 `spring-boot-starter`是包括自动配置支持，日志记录和 YAML 在内的核心启动器。 该应用打包到一个 JAR 文件中。

`resources/application.properties`

```java
spring.main.banner-mode=off
spring.output.ansi.enabled=ALWAYS
logging.pattern.console=%clr(%d{yy-MM-dd E HH:mm:ss.SSS}){blue} %clr(%-5p) %clr(%logger{0}){blue} %clr(%m){faint}%n

app.name=MyApp

```

`application.properties`文件包含应用配置设置。 Spring 具有一些内置的应用属性，我们可以创建自定义属性。 `spring.main.banner-mode`属性是 Spring 内置属性； 我们关闭了 Spring 的标志。 接下来的两行设置了带有颜色支持的日志记录。 `app.name`是我们的自定义属性，其中包含应用名称。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <include resource="org/springframework/boot/logging/logback/base.xml" />
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>
</configuration>

```

在`logback.xml`文件中配置了应用日志记录。 我们设置日志记录级别的级别。 我们不希望我们的输出被不必要的消息所困扰。 `spring-boot-starter`依赖项启用登录日志记录。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.core.env.Environment;

@SpringBootApplication
public class Application implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    @Autowired
    private Environment env;

    @Override
    public void run(String... args) throws Exception {

        logger.info("{}", env.getProperty("JAVA_HOME"));
        logger.info("{}", env.getProperty("app.name"));
    }

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

在`Application`中，我们创建一个 bean，调用其方法并设置 Spring Boot 应用。 `CommandLineRunner`接口指示当`SpringApplication`中包含 bean 时应运行它。 它可以用来在 Spring Boot 中创建命令行应用。

```java
@SpringBootApplication
public class Application implements CommandLineRunner {

```

`@SpringBootApplication`注解启用自动配置和组件扫描。

```java
@Autowired
private Environment env;

```

我们注入`Environment`以获得属性。

```java
logger.info("{}", env.getProperty("JAVA_HOME"));

```

在这里，我们检索`JAVA_HOME`环境变量。

```java
logger.info("{}", env.getProperty("app.name"));

```

也可以使用`Environment`从`application.properties`文件中获取属性：获取`app.name`属性。

```java
$ mvn -q spring-boot:run
...
19-05-23 Thu 18:03:51.558 INFO  Application C:\Users\Jano\AppData\Local\Programs\Java\openjdk-11\
19-05-23 Thu 18:03:51.560 INFO  Application MyApp

```

我们运行该应用。 `-q` Maven 选项关闭 Maven 消息。

在本教程中，我们使用 Spring `Environment`来读取环境变量。 您可能也对相关教程感兴趣： [`SpringApplicationBuilder`教程](/springboot/springapplicationbuilder/)，[在 Spring Boot 中提供静态内容](/springboot/static/)， [Spring Boot `DataSourceBuilder`教程](/springboot/datasourcebuilder/)， [Java 教程](/lang/java/)或列出[所有 Spring Boot 教程](/all/#springboot)。