# Spring Boot 基本注解

> 原文： [http://zetcode.com/springboot/annotations/](http://zetcode.com/springboot/annotations/)

Spring Boot 基本注解教程展示了如何使用基本的 Spring Boot 注解，包括`@Bean`，`@Service`，`@Configuration`，`@Controller`，`@RequestMapping`，`@Repository`，`@Autowired`和`@SpringBootApplication`。

Spring 是用于创建企业应用的流行 Java 应用框架。 Spring Boot 是 Spring 框架发展的下一步。 它有助于以最小的努力创建独立的，基于生产级的 Spring 应用。 它不再使用 XML 配置，并实现了约定而非配置原则。

注解是元数据的一种形式，它提供有关程序的数据，该数据不是程序本身的一部分。 注解对它们注解的代码的操作没有直接影响。

## Spring Boot 基本注解

在示例应用中，我们具有以下 Spring Boot 注解：

*   `@Bean` - 表示方法产生一个由 Spring 管理的 bean。
*   `@Service` - 表示带注解的类是服务类。
*   `@Repository` - 指示带注解的类是存储库，它是数据访问和存储的抽象。
*   `@Configuration` - 表示一个类是可能包含 bean 定义的配置类。
*   `@Controller` - 将类标记为 Web 控制器，能够处理请求。
*   `@RequestMapping` - 将 HTTP 请求及其路径映射到控制器方法。
*   `@Autowired` - 标记要通过 Spring 依赖注入自动装配的构造函数，字段或设置方法。
*   `@SpringBootApplication` - 启用 Spring Boot 自动配置和组件扫描。

`@Component`是 Spring 托管组件的通用构造型。 在自动扫描时，它将类转换为 Spring bean。 当使用基于注解的配置和类路径扫描时，被此注解修饰的类被视为自动检测的候选对象。 `@Repository`，`@Service`和`@Controller`是`@Component`的特殊化，用于更特定的用例。

在示例中，还有 Hibernate `@Entity`，`@Table`，`@Id`和`@GeneratedValue`注解。

## Spring Boot 基本注解示例

以下应用是一个 Spring Boot 应用，它使用 Spring Data JPA 从 H2 数据库返回数据。 该应用使用 FreeMarker 作为模板引擎。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───controller
│   │           │       MyController.java
│   │           ├───model
│   │           │       City.java
│   │           ├───repository
│   │           │       CityRepository.java
│   │           └───service
│   │                   CityService.java
│   │                   ICityService.java
│   └───resources
│       │   application.yaml
│       │   import.sql
│       ├───static
│       │   └───css
│       │           style.css
│       └───templates
│               index.ftl
│               showCities.ftl
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootbasicannotations</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>

        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <scope>runtime</scope>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

这是 Maven 构建文件。 它包含 Freemaker，Spring Data JPA 和 H2 数据库的依赖项。 当 Spring Boot 在`pom.xml`中找到 Freemaker 和 H2 时，它将自动配置它们。 我们可以立即使用它们。

`resources/application.yml`

```java
server:
  servlet:
    context-path: /myapp

spring:
  main:
    banner-mode: "off"
datasource:
    platform: h2

logging:
  level:
    org:
      springframework: ERROR

```

在`application.yml`文件中，我们编写了 Spring Boot 应用的各种配置设置。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "cities")
public class City {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(String name, int population) {
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.id);
        hash = 79 * hash + Objects.hashCode(this.name);
        hash = 79 * hash + this.population;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final City other = (City) obj;
        if (this.population != other.population) {
            return false;
        }
        if (!Objects.equals(this.name, other.name)) {
            return false;
        }
        return Objects.equals(this.id, other.id);
    }

    @Override
    public String toString() {

        var builder = new StringBuilder();
        builder.append("City{id=").append(id).append(", name=")
                .append(name).append(", population=")
                .append(population).append("}");

        return builder.toString();
    }
}

```

这是`City`实体。 每个实体必须至少定义两个注解：`@Entity`和`@Id`。 `@Entity`注解指定该类是一个实体，并映射到数据库表。 `@Table`注解指定要用于映射的数据库表的名称。 `@Id`注解指定了实体的主键，`@GeneratedValue`为主键的值提供了生成策略的规范。

`resources/import.sql`

```java
INSERT INTO cities(name, population) VALUES('Bratislava', 432000);
INSERT INTO cities(name, population) VALUES('Budapest', 1759000);
INSERT INTO cities(name, population) VALUES('Prague', 1280000);
INSERT INTO cities(name, population) VALUES('Warsaw', 1748000);
INSERT INTO cities(name, population) VALUES('Los Angeles', 3971000);
INSERT INTO cities(name, population) VALUES('New York', 8550000);
INSERT INTO cities(name, population) VALUES('Edinburgh', 464000);
INSERT INTO cities(name, population) VALUES('Berlin', 3671000);

```

模式是由 Hibernate 自动创建的。 之后，将执行`import.sql`文件以将数据填充到表中。

`com/zetcode/repository/CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

@Repository
public interface CityRepository extends CrudRepository<City, Long> {

}

```

`@Repository`注解用于定义存储库。

`com/zetcode/service/ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import java.util.List;

public interface ICityService {

    List<City> findAll();
}

```

`ICityService`提供了获取所有城市的契约方法。

`com/zetcode/service/CityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import com.zetcode.repository.CityRepository;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CityService implements ICityService {

    @Autowired
    private CityRepository cityRepository;

    @Override
    public List<City> findAll() {

        return (List<City>) cityRepository.findAll();

    }
}

```

`@Service`注解将`CityService`声明为服务类； 提供业务服务的类。 `@Autowired`注解标记要插入`CityRepository`的`cityRepository`字段。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.service.ICityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.HashMap;
import java.util.Map;

@Controller
public class MyController {

    @Autowired
    private ICityService cityService;

    @RequestMapping("/")
    public String index(Model model) {

        return "index";
    }

    @RequestMapping("/cities")
    public ModelAndView showCities() {

        var cities = cityService.findAll();

        Map<String, Object> params = new HashMap<>();
        params.put("cities", cities);

        return new ModelAndView("showCities", params);
    }
}

```

`@Controller`注解将一个类标记为 Web 控制器。 `@RequestMapping`将 HTTP 请求及其路径映射到控制器方法。 在第二种情况下，它将`/cities` URL 映射到`showCities()`方法。

`resources/templates/index.ftl`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <a href="cities">Show cities</a>
    </body>
</html>

```

这是`index.ftl`模板文件。 它包含用于创建显示所有城市的请求的链接。

`resources/templates/showCities.ftl`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Cities</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <link rel="stylesheet" href="css/style.css">
    </head>
    <body>
        <h2>List of cities</h2>

        <table>
            <tr>
                <th>Id</th>
                <th>Name</th>
                <th>Population</th>
            </tr>

            <#list cities as city>
                <tr>
                    <td>${city.id}</td>
                    <td>${city.name}</td>
                    <td>${city.population}</td>
                </tr>
            </#list>
        </table>
    </body>
</html>

```

这是`showCities.ftl`模板文件。 它使用 FreeMarker `#list`宏显示所有城市对象。

`resources/static/css/style.css`

```java
h2 {color: blue}

td:nth-child(3) {
    text-align: right;
}

```

这是`style.css`模板文件。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`@SpringBootApplication`启用自动配置和组件扫描。

```java
$ mvn spring-boot:run 

```

我们运行该应用并定位到`localhost:8080/myapp`地址。

在本教程中，我们介绍了一些基本的 Spring Boot 注解。

列出[所有 Spring Boot 教程](/all/#springboot)。