# Spring Boot `@RequestParam`教程

> 原文： [http://zetcode.com/springboot/requestparam/](http://zetcode.com/springboot/requestparam/)

在本教程中，我们将在控制器中使用`@RequestParam`注解来读取请求参数。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## Spring `@RequestParam`

`@RequestParam`是一个 Spring 注解，用于将 Web 请求参数绑定到方法参数。

它具有以下可选元素：

*   `defaultValue` - 当没有提供请求参数或值为空时用作备用
*   `name` - 要绑定到的请求参数的名称
*   `required` - 告诉参数是否为必需
*   `value` - 名称的别名

## Spring `@RequestParam`示例

以下示例创建一个使用`@RequestParam`的 Spring Boot Web 应用。 我们有一个带有两个标签的 HTML 表单：文本输入和复选框。 这两个标记创建请求参数，该参数在控制器中通过`@RequestParam`读取。

```java
$ tree
.
├── pom.xml
└── src
    ├── main
    │   ├── java
    │   │   └── com
    │   │       └── zetcode
    │   │           ├── Application.java
    │   │           └── controller
    │   │               └── MyController.java
    │   └── resources
    │       └── static
    │           └── index.html
    └── test
        └── java

```

这是 Spring Boot 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>requestparamex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>
    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-devtools</artifactId>
            <optional>true</optional>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

在 Maven 构建文件中，我们有`spring-boot-starter-web`，它是使用 Spring MVC 构建 Web 应用的入门。 它使用 Tomcat 作为默认的嵌入式容器。 `spring-boot-devtools`是在开发 Spring Boot 应用时有用的构件。 它允许自动重启或实时重新加载应用。 该应用打包到一个 JAR 文件中。

`com/zetcode/MyController.java`

```java
package com.zetcode.controller;

import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
public class MyController {

    @RequestMapping(path="/message", produces=MediaType.TEXT_PLAIN_VALUE)
    @ResponseBody
    public String processForm(@RequestParam(defaultValue="Guest") String name,
                              @RequestParam(required = false) String adult) {

        var greet = "on".equals(adult) ? "Good morning" : "Hi";
        var message = String.format("%s %s!", greet, name);

        return message;
    }
}

```

控制器处理 HTML 表单。 它从请求中读取两个参数。

```java
@Controller
public class MyController {

```

在 Spring 中，控制器类使用`@Controller`注解进行注解。

```java
@RequestMapping(path="/message", produces=MediaType.TEXT_PLAIN_VALUE)
@ResponseBody

```

`processForm()`方法映射到`/message`路径并返回纯文本。 `@ResponseBody`注解指示方法返回值已绑定到 Web 响应正文。

```java
public String processForm(@RequestParam(defaultValue="Guest") String name,
        @RequestParam(required = false) String adult) {

```

使用`@RequestParam`注解，我们将请求参数绑定到方法变量。 如果参数不可用（文本输入为空），则`defaultValue`选项将提供默认值。 `required`选项表明该参数是必需的。 该方法重新调整字符串。

```java
var greet = "on".equals(adult) ? "Good morning" : "Hi";
var message = String.format("%s %s!", greet, name);

return message;

```

我们构建消息并返回。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html lang="en">
    <head>
        <title>Home page</title>
        <meta charset="UTF-8"/>
        <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    </head>
    <body>

        <form action="message">

            <div>
                <label>Name:</label>
                <input type="text" name="name">
            </div>

            <div>
                <label><input type="checkbox" name="adult">Adult</label>
            </div>

            <button type="submit">Submit</button>

        </form>
    </body>
</html>

```

`index.html`文件是主页。 该文件位于`src/main/resources/static`目录中，Spring Boot 在该目录中需要静态资源，例如 HTML 或 CSS 文件。 我们有一个带有输入文本和复选框标签的简单 HTML 表单。

```java
<form action="message">

```

`action`选项包含在控制器方法映射中使用的字符串。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。 `@SpringBootApplication`注解启用自动配置和组件扫描。

```java
$ mvn spring-boot:run

```

应用运行后，我们可以导航到`localhost:8080`。

在本教程中，我们使用 Spring Boot 框架创建了 Web 应用。 我们已经演示了`@RequestParam`的用法。 您可能也对相关教程感兴趣： [Spring Boot `@PathVariable`教程](/springboot/pathvariable/)， [Spring Boot `@ResponseBody`教程](/springboot/responsebody/)， [Java 教程](/lang/java/)或列出[所有 Spring Boot 教程](/all/#springboot)。