# Spring Boot `@Bean`

> 原文： [http://zetcode.com/articles/springbootbean/](http://zetcode.com/articles/springbootbean/)

在 Spring Boot `@Bean`教程中，我们使用`@Bean`注解在 Spring Boot 框架中创建一个简单的 Bean。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可帮助您以最少的精力创建独立的，生产级的基于 Spring 的应用。

## Spring `@Bean`注解

Spring `@Bean`注解告诉一个方法产生一个由 Spring 容器管理的 bean。 它是方法级别的注解。 在 Java 配置（`@Configuration`）期间，将执行该方法并将其返回值注册为`BeanFactory`中的 Bean。

## Spring Boot `@Bean`示例

Spring 核心容器创建并管理 Bean。 在以下应用中，我们演示如何创建带有`@Bean`注解的 Spring bean。 该应用是命令行 Spring Boot 应用。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │               Application.java
│   │               AppName.java
│   │
│   └───resources
│           application.properties
│           logback.xml
│
└───test
    └───java

```

这是 Spring Boot 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootbean</artifactId>
    <version>1.0-SNAPSHOT</version>

    <packaging>jar</packaging>
    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

这是 Maven 构建文件。 `spring-boot-starter`是包括自动配置支持，日志记录和 YAML 在内的核心启动器。 该应用打包到一个 JAR 文件中。

`com/zetcode/AppName.java`

```java
package com.zetcode;

interface AppName {

    String getName();
}

```

我们有一个定义契约的简单接口。 它用于创建返回应用名称的匿名类。

`resources/application.properties`

```java
spring.main.banner-mode=off
app.name=SpringBootBean

```

`application.properties`文件包含应用配置设置。 有一些内置的应用属性，我们可以创建自定义属性。 `spring.main.banner-mode`属性是 Spring 内置属性； 我们关闭了 Spring 的标志。 `app.name`是我们的自定义属性，其中包含应用名称。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <include resource="org/springframework/boot/logging/logback/base.xml" />
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>
</configuration>

```

在`logback.xml`文件中，我们配置应用日志记录。 我们将日志记录级别设置为`ERROR`。 这样，我们的输出就不会充满不必要的信息。 `spring-boot-starter`依赖项启用登录日志记录。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

@SpringBootApplication
public class Application implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    @Autowired
    private AppName appName;

    @Bean
    public AppName getAppName(@Value("${app.name}") String appName) {

        return () -> appName;
    }

    @Override
    public void run(String... args) throws Exception {

        logger.info("Application name: {}", appName.getName());
    }

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

在`Application`中，我们创建一个 bean，调用其方法并设置 Spring Boot 应用。 `CommandLineRunner`接口指示当`SpringApplication`中包含 bean 时应运行它。 它可以用来在 Spring Boot 中创建命令行应用。

```java
@SpringBootApplication
public class Application implements CommandLineRunner {

```

`@SpringBootApplication`注解启用自动配置和组件扫描。

```java
@Autowired
private AppName appName;

```

使用`@Autowired`注解，我们将`AppName` bean 注入到字段中。

```java
@Bean
public AppName getAppName(@Value("${app.name}") String appName) {

    return () -> appName;
}

```

在这里，我们创建`AppName` bean； 该 bean 由 Spring 容器管理。 尽管`@Component`注解用于装饰由 Spring 扫描自动检测的类，但`@Bean`注解用于显式声明 bean 创建。

`@Value`注解用于将`app.name`属性的值设置为`appName`参数。

```java
logger.info("Application name: {}", appName.getName());

```

我们调用 bean 的`getName()`方法。

我们使用`mvn spring-boot:run`运行该应用。

在本教程中，我们创建了带有`@Bean`注解的 Spring bean。 您可能也对相关教程感兴趣： [Spring Boot `@Lazy`教程](/springboot/lazybean/)， [Java 教程](/lang/java/)或列出 [Spring Boot 教程](/all/#springboot)。