# Spring Boot `@ResponseStatus`

> 原文： [http://zetcode.com/springboot/responsestatus/](http://zetcode.com/springboot/responsestatus/)

Spring Boot `@ResponseStatus`教程展示了如何在 Spring 应用中使用`@ResponseStatus`注解。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## `@ResponseStatus`

`@ResponseStatus`用应返回的状态代码和原因消息标记方法或异常类。 调用处理器方法时或抛出指定的异常时，状态代码将应用于 HTTP 响应。 它会覆盖通过其他方式设置的状态信息，例如`ResponseEntity`或`redirect:`。

## Spring Boot `@ResponseStatus`示例

在以下应用中，我们演示`@ResponseStatus`注解的用法。 该应用模拟用于通过其 ID 检索订单的表单。 尝试查找 ID 大于 500 的订单将引发异常。 由于此异常，将显示一个自定义错误页面。

```java
pom.xml
src
├── main
│   ├── java
│   │   └── com
│   │       └── zetcode
│   │           ├── Application.java
│   │           ├── controller
│   │           │   └── MyController.java
│   │           └── exception
│   │               └── OrderNotFoundException.java
│   └── resources
│       ├── static
│       │   └── index.html
│       └── templates
│           └── error.ftl
└── test
    └── java

```

这是 Spring 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>springbootresponsestatusex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>
        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>
    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

这是 Maven `pom.xml`文件。 `spring-boot-starter-freemarker`是 Freemarker 模板引擎的依赖项； `spring-boot-maven-plugin`将 Spring 应用打包到可执行的 JAR 或 WAR 归档文件中。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.exception.OrderNotFoundException;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
public class MyController {

    @RequestMapping(value = "/orders/{id}")
    @ResponseBody
    public String getOrder(@PathVariable("id") long id) {

        if (id < 0 || id > 500) {

            var message = String.format("Order %d not found", id);
            throw new OrderNotFoundException(message);
        }

        var message = String.format("Returning order %d", id);

        return message;
    }
}

```

`MyController`的`getOrder()`方法响应客户端请求。 它使用`@PathVariable`从路径读取顺序 ID。

```java
if (id < 0 || id > 500) {

    var message = String.format("Order %d not found", id);
    throw new OrderNotFoundException(message);
}

```

对于无效订单（`id < 0`）和大于 500 的订单，我们抛出`OrderNotFoundException`异常。 这是订单系统的简单模拟。

```java
var message = String.format("Returning order %d", id);

```

对于其他订单 ID，我们返回一条消息，指示已找到并返回了该订单。

`com/zetcode/exception/OrderNotFoundException.java`

```java
package com.zetcode.exception;

import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ResponseStatus;

@ResponseStatus(value = HttpStatus.NOT_FOUND, reason = "No such order")
public class OrderNotFoundException extends RuntimeException {

    public OrderNotFoundException(String message) {

        super(message);
    }
}

```

我们有一个自定义`OrderNotFoundException`。 它以`@ResponseStatus`注解装饰。 该值设置为`HttpStatus.NOT_FOUND`，并且原因消息显示“无此顺序”。 此信息将在错误页面中使用。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <a href="/orders/505/">Get order with Id 505</a>
    </body>
</html>

```

这是主页。 它包含一个链接，用于查找带有 ID 505 的订单。该文件位于`src/main/resources/static`目录中。

`resources/templates/error.ftl`

```java
<html>
    <head>
        <title>Error page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>

    <body>
        <div>
            <h1>Error occurred</h1>

            <p>${status}: ${error} - ${message}</p>
        </div>
    </body>
</html>

```

`error.ftl`是通用错误页面。 这是 Freemarker 模板文件，显示状态，错误和原因消息。 这些值是通过`@ResponseStatus`设置的。 它位于`src/main/resources/templates`目录中。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。

![Error page](img/e6fd3ba1a90dd89f6f09d5a726229210.jpg)

图：错误页面

在本教程中，我们展示了如何在 Spring 应用中使用`@ResponseStatus`注解。 您可能也对相关教程感兴趣： [Spring Boot `ResponseEntity`教程](/springboot/responseentity/)， [Spring Boot `@ExceptionHandler`教程](/springboot/exceptionhandler/)， [Java 教程](/lang/java/)或列出[所有 Spring Boot 教程](/all/#springboot)。