# Spring Boot `@Controller`

> 原文： [http://zetcode.com/springboot/controller/](http://zetcode.com/springboot/controller/)

Spring Boot `@Controller`教程显示了如何在 Spring 应用中使用`@Controller`注解来构建 Web 控制器。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## Spring MVC

Spring MVC 是基于 Servlet API 构建的原始 Web 框架。 它基于流行的 MVC 设计模式。 MVC（模型-视图-控制器）是一种软件体系结构模式，它将应用分为三个区域：模型，视图和控制器。 该模型表示一个携带数据的 Java 对象。 该视图表示模型包含的数据的可视化。 控制器控制数据流入模型对象并在数据更改时更新视图。 它使视图和模型分离。

Spring Framework 5.0 引入了一个名为 Spring WebFlux 的并行反应式栈 Web 框架。

## `@Controller`

`@Controller`注解指示已注解的类是控制器。 它是`@Component`的特化，可通过类路径扫描自动检测。 它通常与基于`@RequestMapping`注解的注解处理器方法结合使用。 `@RestController`是用于创建 Restful 控制器的同级便利注解。

## Spring Boot `@Controller`示例

在以下应用中，我们演示`@Controller`的用法。 该应用将当前数据和时间返回给客户端。

```java
$ tree
.
├── pom.xml
└── src
    ├── main
    │   ├── java
    │   │   └── com
    │   │       └── zetcode
    │   │           ├── Application.java
    │   │           └── controller
    │   │               └── MyController.java
    │   └── resources
    │       ├── static
    │       │   └── index.html
    │       └── templates
    │           └── showMessage.ftl
    └── test
        └── java

```

这是 Spring 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>controllerex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

这是 Maven `pom.xml`文件。 `spring-boot-starter-parent`是父 POM，它为使用 Maven 构建的应用提供依赖关系和插件管理。 `spring-boot-starter-freemarker`是 Freemarker 模板引擎的依赖项。 此依赖项还将在项目中包含 Spring MVC。 `spring-boot-maven-plugin`将 Spring 应用打包到可执行的 JAR 或 WAR 归档文件中。

`com/zetcode/MyController.java`

```java
package com.zetcode.controller;

import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;

@Controller
public class MyController {

    @RequestMapping(value = "/getDateAndTime")
    public ModelAndView getDateAndTime() {

        var now = LocalDateTime.now();
        var dtf = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
        var date_time = dtf.format(now);

        var params = new HashMap<String, Object>();
        params.put("date_time", date_time);

        return new ModelAndView("showMessage", params);
    }
}

```

这是`MyController`。 它响应来自客户端的请求。 它找出当前日期和时间，并将处理过程解析为`showMessage.ftl`模板，并将其传递给数据。

```java
@Controller
public class MyController {

```

`MyController`带有`@Controller`注解。

```java
@RequestMapping(value = "/getDateAndTime")
public ModelAndView getDateAndTime() {

```

`getDateAndTime()`方法映射到`getDateAndTime` URL 模式； 它返回`ModelAndView`，它是 Web MVC 框架中 Model 和 View 的持有人。

```java
var now = LocalDateTime.now();
var dtf = DateTimeFormatter.ofPattern("yyyy/MM/dd HH:mm:ss");
var date_time = dtf.format(now);

```

我们获取并格式化本地日期和时间。

```java
var params = new HashMap<String, Object>();
params.put("date_time", date_time);

```

日期和时间字符串将添加到参数映射中。

```java
return new ModelAndView("showMessage", params);

```

我们返回`ModelAndView`。 由于 POM 文件中存在 Freemarker 依赖性，因此 Spring 将处理解析为`showMessage.ftl`模板文件，并将`params`对象传递给它。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <p>
            <a href="getDateAndTime">Get date and time</a>
        </p>
    </body>
</html>

```

这是主页。 它包含一个调用 Spring 控制器的链接。 它是静态资源，位于预定义的`src/main/resources/static`目录中。

`resources/templates/showMessage.ftl`

```java
<!DOCTYPE html>
<html>
<head>
    <title>Show message</title>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body>

Date and time: ${date_time}    

</body>
</html>

```

`showMessage.ftl`是一个 Freemarker 模板文件。 它位于预定义的`src/main/resources/templates`目录中。 它使用`${}`语法输出日期和时间。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。 `@SpringBootApplication`注解启用自动配置和组件扫描。 在扫描过程中，将查找`@Controller`注解，并从`MyController`类创建一个 Spring bean。

```java
$ mvn spring-boot:run 

```

应用运行后，我们可以导航到`localhost:8080`。

在本教程中，我们展示了如何在 Spring 应用中使用`@Controller`注解。 您可能也对相关教程感兴趣： [Spring Boot `@RestController`教程](/springboot/restcontroller/)， [Java Servlets 教程](/articles/javaservlet/)， [Spring Boot `@ResponseStatus`教程](/springboot/responsestatus/)， [Spring Boot `@ExceptionHandler`教程](/springboot/exceptionhandler/)， [Spring Boot 上传文件](/springboot/uploadfile/)， [Spring Boot `@Component`教程](/springboot/component/)， [Spring Boot `@RequestParam`教程](/springboot/requestparam/)， [Java 教程](/lang/java/)。