# Spring `@RequestHeader`教程

> 原文： [http://zetcode.com/spring/requestheader/](http://zetcode.com/spring/requestheader/)

Spring `@RequestHeader`教程展示了如何使用`@RequestHeader`注解将方法参数绑定到请求标头。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@RequestHeader`

`@RequestHeader`注释将请求标头值绑定到方法参数。 如果方法参数为`Map`，`MultiValueMap<String, String>`或`HttpHeaders`，则将使用所有标头名称和值填充映射。

## Spring `@RequestHeader`示例

该应用将请求正文标头绑定到方法参数。 使用`curl`工具创建请求。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           ├───config
│   │           │       MyWebInitializer.java
│   │           │       WebConfig.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springrequestheader</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>

        </plugins>
    </build>
</project>

```

我们在`pom.xml`中声明必要的依赖关系。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

@Configuration
public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {

        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {

        return new String[]{"/"};
    }
}

```

`MyWebInitializer`注册 Spring `DispatcherServlet`，它是 Spring Web 应用的前端控制器。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
@EnableWebMvc
@ComponentScan(basePackages = {"com.zetcode"})
public class WebConfig implements WebMvcConfigurer {

}

```

`WebConfig`通过`@EnableWebMvc`启用 Spring MVC 注解，并为`com.zetcode`包配置组件扫描。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;
package com.zetcode.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@RestController
public class MyController {

    private static final Logger logger = LoggerFactory.getLogger(MyController.class);

    @GetMapping(value = "/agent")
    @ResponseStatus(value = HttpStatus.OK)
    public void client(@RequestHeader(value="User-Agent") String userAgent) {

        logger.info("User agent is: {}", userAgent);
    }

    @GetMapping(value = "/all")
    @ResponseStatus(value = HttpStatus.OK)
    public void all(@RequestHeader Map<String, String> headers) {

        logger.info("All headers: {}", headers);
    }
}

```

我们有两个映射。 第一个映射确定用户代理，第二个映射找出所有发送的请求标头。

```java
public void client(@RequestHeader(value="User-Agent") String userAgent) {

```

使用`@RequestHeader`的 value 参数，我们查找特定的标头； 在我们的例子中是`User-Agent`。

```java
public void all(@RequestHeader Map<String, String> headers) {

```

提供地图时，我们会检索所有标头。

```java
$ mvn jetty:run

```

我们启动服务器。

```java
$ curl localhost:8080/agent

```

我们创建对第一个映射的请求。

```java
08:26:29.926 INFO  com.zetcode.controller.MyController - User agent is: curl/7.55.1    

```

我们得到这个日志。

```java
$ curl localhost:8080/all

```

我们调用第二个映射。

```java
08:27:26.564 INFO  com.zetcode.controller.MyController - All headers: {User-Agent=curl/7.55.1, Accept=*/*, Host=localhost:8080}

```

我们记录了三个标头。

在本教程中，我们使用`@RequestHeader`注解将请求标头绑定到方法参数。

您可能也对这些相关教程感兴趣： [Spring @GetMapping 教程](/spring/getmapping/)， [Spring @RequestBody 教程](/spring/requestbody/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。