# Spring `MessageSource`教程

> 原文： [http://zetcode.com/spring/messagesource/](http://zetcode.com/spring/messagesource/)

Spring `MessageSource`教程展示了如何在 Spring 应用中使用`MessageSource`转换消息。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `MessageSource`

`MessageSource`用于解析消息，并支持消息的参数化和国际化。 Spring 包含两个内置的`MessageSource`实现：`ResourceBundleMessageSource`和`ReloadableResourceBundleMessageSource`。 后者能够重新加载消息定义，而无需重新启动虚拟机。

## Spring `MessageSource`示例

以下应用包含英语和德语消息。 它使用内置的`ResourceBundleMessageSource`。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───config
│   │                   AppConfig.java
│   └───resources
│       │   logback.xml
│       └───messages
│               label.properties
│               label_de.properties
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>messagesource</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/messages/labels.properties`

```java
l1=Earth
l2=Hello {0}, how are you?

```

这些是英文信息。 第二个属性接收一个参数。

`resources/messages/labels_de.properties`

```java
l1=Erde
l2=Hallo {0}, wie geht's?

```

这些是德语信息。

`com/zetcode/config/AppConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.ResourceBundleMessageSource;

@Configuration
public class AppConfig {

    @Bean
    public ResourceBundleMessageSource messageSource() {

        var source = new ResourceBundleMessageSource();
        source.setBasenames("messages/label");
        source.setUseCodeAsDefaultMessage(true);

        return source;
    }
}

```

`AppConfig`配置`ResourceBundleMessageSource`。 `setBasenames()`告诉在哪里查找消息定义。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;

import java.util.Locale;

@ComponentScan(basePackages = "com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    @Autowired
    private MessageSource messageSource;

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);

        var app = ctx.getBean(Application.class);
        app.run();

        ctx.close();
    }

    public void run() {

        logger.info("Translated messages:");
        logger.info("{}", messageSource.getMessage("l1",
                null, Locale.GERMAN));
        logger.info("{}", messageSource.getMessage("l1",
                null, Locale.ENGLISH));

        logger.info("Translated parameterized messages:");
        logger.info("{}", messageSource.getMessage("l2",
                new Object[] {"Paul Smith"}, Locale.GERMAN));
        logger.info("{}", messageSource.getMessage("l2",
                new Object[] {"Paul Smith"}, Locale.ENGLISH));
    }
}

```

该应用将普通消息和参数化消息打印到控制台。

```java
@Autowired
private MessageSource messageSource;

```

我们注入在`AppConfig`中生成的`MessageSource`。

```java
logger.info("{}", messageSource.getMessage("l1",
    null, Locale.GERMAN));

```

`getMessage()`将属性名称作为第一个参数。 第二个参数是`null`，因为消息没有任何参数。 第三个参数是语言环境。

```java
logger.info("{}", messageSource.getMessage("l2",
    new Object[] {"Paul Smith"}, Locale.GERMAN));

```

在这里，我们还为消息提供了一个参数。

```java
$ mvn -q exec:java
22:08:27.984 INFO  com.zetcode.Application - Translated messages:
22:08:27.984 INFO  com.zetcode.Application - Erde
22:08:27.984 INFO  com.zetcode.Application - Earth
22:08:27.984 INFO  com.zetcode.Application - Translated parameterized messages:
22:08:27.984 INFO  com.zetcode.Application - Hallo Paul Smith, wie gehts?
22:08:27.984 INFO  com.zetcode.Application - Hello Paul Smith, how are you?

```

我们运行该应用。

在本教程中，我们展示了如何在 Spring 应用中使用`ResourceBundleMessageSource`。

您可能也对这些相关教程感兴趣： [Java `ResourceBundle`教程](/java/resourcebundle/)， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。