# Spring `BeanFactoryPostProcessor`教程

> 原文： [http://zetcode.com/spring/beanfactorypostprocessor/](http://zetcode.com/spring/beanfactorypostprocessor/)

Spring `BeanFactoryPostProcessor`教程展示了如何使用`BeanFactoryPostProcessor`来修改应用上下文的 Bean 定义。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `BeanFactoryPostProcessor`

在实际创建 bean 之前，Spring `BeanFactoryPostProcessor`会处理 bean 的 bean 定义或配置元数据。 使用`BeanFactoryPostProcessor`，我们可以修改现有的 bean 定义或创建新的 bean 定义。

Spring 提供了`BeanFactoryPostProcessor`的一些内置实现。 例如，`PropertyPlaceholderConfigurer`是一种后处理器，可以从外部文件读取属性。

## Spring `BeanFactoryPostProcessor`示例

以下应用使用`BeanFactoryPostProcessor`添加新的 bean 定义。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           │
│   │           └───config
│   │                   AppConfig.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>beanfactorypostprocessorex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency> 

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/config/AppConfig.java`

```java
package com.zetcode.config;

import org.springframework.beans.factory.config.BeanFactoryPostProcessor;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Configuration
public class AppConfig {

    @Bean
    public static BeanFactoryPostProcessor beanFactoryPostProcessor() {
        return factory -> {

            var registry = (BeanDefinitionRegistry) factory;

            registry.registerBeanDefinition("myBean",
                    BeanDefinitionBuilder.genericBeanDefinition(String.class)
                            .addConstructorArgValue("This is myBean")
                            .getBeanDefinition()
            );
        };
    }
}

```

`AppConfig`是应用配置类。 使用`@Bean`创建一个`BeanFactoryPostProcessor`，它注册一个内置的`java.lang.String`类型的新简单 bean。 新 bean 已向`registerBeanDefinition()`注册。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.GenericXmlApplicationContext;

public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new GenericXmlApplicationContext("my-beans.xml");

        logger.info("{}", ctx.getBean("myBean"));

        ctx.close();
    }
}

```

在`Application`中，我们获取已注册的 bean 并进行打印。

```java
$ mvn -q exec:java
18:53:19.233 INFO  com.zetcode.Application - This is myBean

```

我们运行该应用。

在本教程中，我们展示了如何使用`BeanFactoryPostProcessor`注册新 bean。

您可能也对这些相关教程感兴趣： [Spring `@Qualifier`注释教程](/spring/qualifier/)， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring C 命名空间教程](/spring/cnamespace/)， [Spring `context:property-placeholder`教程](/spring/propertyplaceholder/)， [Spring `BeanDefinitionBuilder`教程](/spring/beandefinitionbuilder/)， [Spring bean 参考教程](/spring/beanreference/)和 [Java 教程](/lang/java/)。