# Spring `@Bean`注释教程

> 原文： [http://zetcode.com/spring/bean/](http://zetcode.com/spring/bean/)

Spring `@Bean`注释教程展示了如何使用`@Bean`注释在 Java 配置类中声明 bean。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@Bean`

`@Bean`注释指示带注释的方法产生要由 Spring 容器管理的 bean。 它是`<bean/>` XML 标签的直接模拟。 `@Bean`支持`<bean/>`提供的大多数属性，例如：`init-method`，`destroy-method`，`autowiring`，`lazy-init`，`dependency-check`，`depends-on`，`scope`。

## Spring `@Bean`示例

该应用生成带有`@Bean`注解的 Spring 托管 bean。 它还为 bean 提供了一些别名。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───bean
│   │           │       HelloMessage.java
│   │           └───config
│   │                   AppConfig.java
│   └───resources
│           logback.xml
│           messages.properties
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>beanannotation</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>      

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/message.properties`

```java
motd="Hello there!"

```

`message.properties`包含`day`属性的消息，由我们的`HelloMessage` bean 使用。 这为应用提供了更大的灵活性，并避免了将消息硬编码为 Java 代码。

`com/zetcode/bean/HelloMessage.java`

```java
package com.zetcode.bean;

public class HelloMessage {

    private String message;

    public HelloMessage(String message) {

        this.message = message;
    }

    public String getMessage() {

        return message;
    }
}

```

`HelloMessage` bean 是使用`@Bean`注释方法创建的。

`com/zetcode/config/AppCofig.java`

```java
package com.zetcode.config;

import com.zetcode.bean.HelloMessage;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

@Configuration
@PropertySource(value="messages.properties")
public class AppConfig {

    @Value("${motd}")
    private String message;

    @Bean(name={"myMessage", "motd"})
    public HelloMessage helloMessageProducer() {

        var helloMessage = new HelloMessage(message);

        return helloMessage;
    }
}

```

我们在`AppConfig`中定义了`HelloMessage`生产者。

```java
@Configuration
@PropertySource(value="messages.properties")
public class AppConfig {

```

使用`@Configuration`，我们声明`AppConfig`是配置类。 `@PropertySource`注解允许我们通过`@Value`轻松使用`messages.properties`文件中的属性。

```java
@Value("${motd}")
private String message;

```

我们将`motd`属性注入到`message`属性中。

```java
@Bean(name={"myMessage", "motd"})
public HelloMessage helloMessageProducer() {

    var helloMessage = new HelloMessage(message);

    return helloMessage;
}

```

`helloMessageProducer()`产生一个新的`HelloMessage` bean。 它从外部属性获取消息。 `@Bean`注释使`HelloMessage` bean 由 Spring 生产和管理。 另外，我们给 Bean 两个别名。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.bean.HelloMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;

@ComponentScan(basePackages = "com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);

        var msgBean1 = ctx.getBean(HelloMessage.class);
        logger.info("{}", msgBean1.getMessage());

        var msgBean2 = (HelloMessage) ctx.getBean("myMessage");
        logger.info("{}", msgBean2.getMessage());

        var msgBean3 = (HelloMessage) ctx.getBean("motd");
        logger.info("{}", msgBean3.getMessage());

        ctx.close();
    }
}

```

该应用带有`@ComponentScan`注释。 `basePackages`选项告诉 Spring 在`com/zetcode`包及其子包中查找组件。

```java
var ctx = new AnnotationConfigApplicationContext(Application.class);

```

`AnnotationConfigApplicationContext`是 Spring 独立应用上下文。 它接受带注释的`Application`作为输入； 因此启用了扫描。

```java
var msgBean1 = ctx.getBean(HelloMessage.class);
logger.info("{}", msgBean1.getMessage());

```

我们通过其类型来获取 bean。

```java
var msgBean2 = (HelloMessage) ctx.getBean("myMessage");
logger.info("{}", msgBean2.getMessage());

var msgBean3 = (HelloMessage) ctx.getBean("motd");
logger.info("{}", msgBean3.getMessage());

```

在这里，我们通过别名获得相同的 bean。

```java
$ mvn -q exec:java
14:39:29.324 INFO  com.zetcode.Application - "Hello there!" 
14:39:29.324 INFO  com.zetcode.Application - "Hello there!" 
14:39:29.324 INFO  com.zetcode.Application - "Hello there!" 

```

我们运行该应用。

在本教程中，我们使用了`@Bean`注释来生成托管的 Spring bean。

您可能也会对这些相关教程感兴趣： [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Spring `@Autowired`教程](/spring/autowired/)， [Java 教程](/lang/java/)，或列出[所有 Spring 教程](/all/#spring)。