# Spring `@PropertySource`注释教程

> 原文： [http://zetcode.com/spring/propertysource/](http://zetcode.com/spring/propertysource/)

Spring `@PropertySource`注解教程显示了如何使用`@PropertySource`注解将属性包括到环境中以及如何使用`@Value`注入属性。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@PropertySource`

`@PropertySource`是方便的注释，用于将`PropertySource`包含到 Spring 的环境中，并允许通过`@Value`将属性注入类属性中。 （`PropertySource`是代表来自特定来源的一组属性对的对象。）

`@PropertySource`与`@Configuration`一起使用。

## Spring `@PropertySource`示例

该应用使用 Spring 的`@PropertySource`将`application.properties`文件中的属性包含到环境中，并将它们注入类属性中。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───config
│   │                   AppConfig.java
│   └───resources
│           application.properties
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>propertysource</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency> 

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/application.properties`

```java
app.name=My application
app.version=1.1

```

`application.properties`文件中有两个属性。

`com/zetcode/config/AppConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

@Configuration
@PropertySource(value = "application.properties", ignoreResourceNotFound = true)
public class AppConfig {

}

```

`AppConfig`是应用配置类。 `@PropertySource`将属性从`application.properties`注入到 Spring 的环境中。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.core.env.Environment;

@ComponentScan(basePackages = "com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    @Autowired
    private Environment env;

    @Value("${app.name}")
    private String appName;

    @Value("${app.version}")
    private String appVersion;

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);
        var app = ctx.getBean(Application.class);

        app.run();

        ctx.close();
    }

    private void run() {

        logger.info("From Environment");
        logger.info("Application name: {}", env.getProperty("app.name"));
        logger.info("Application version: {}", env.getProperty("app.version"));

        logger.info("Using @Value injection");
        logger.info("Application name: {}", appName);
        logger.info("Application version: {}", appVersion);
    }
}

```

在`Application`中，我们使用两种方法获得属性。

```java
@Autowired
private Environment env;

```

我们注入`Environment`。 我们可以使用`getProperty()`方法检索属性。

```java
@Value("${app.name}")
private String appName;

@Value("${app.version}")
private String appVersion;

```

我们将带有`@Value`注解的属性注入到属性中。

```java
logger.info("From Environment");
logger.info("Application name: {}", env.getProperty("app.name"));
logger.info("Application version: {}", env.getProperty("app.version"));

```

检索属性的第一种方法是使用`getProperty()`方法从`Environment`。

```java
logger.info("Using @Value injection");
logger.info("Application name: {}", appName);
logger.info("Application version: {}", appVersion);

```

第二种方法是使用注入的属性。

```java
$ mvn -q exec:java
15:00:20.653 INFO  com.zetcode.Application - From Environment 
15:00:20.668 INFO  com.zetcode.Application - Application name: My application 
15:00:20.668 INFO  com.zetcode.Application - Application version: 1.1 
15:00:20.668 INFO  com.zetcode.Application - Using @Value injection 
15:00:20.668 INFO  com.zetcode.Application - Application name: My application 
15:00:20.668 INFO  com.zetcode.Application - Application version: 1.1 

```

我们运行该应用。

在本教程中，我们展示了如何使用`@PropertySource`注释在 Spring 应用中方便地使用属性。

您可能也对这些相关教程感兴趣： [Spring `@Qualifier`注释教程](/spring/qualifier/)， [Spring `@Configuration`注释教程](/spring/configuration/)， [Spring `context:property-placeholder`教程](/spring/propertyplaceholder/)，Java 教程，或列出[所有 Spring 教程](/all/#spring)。