# Spring `@Configuration`教程

> 原文： [http://zetcode.com/spring/configuration/](http://zetcode.com/spring/configuration/)

Spring `@Configuration`注释教程展示了如何使用`@Configuration`注释配置 Spring 应用。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@Configuration`

`@Configuration`注释用于基于 Spring 注释的配置。 `@Configuration`是标记注释，指示类声明了一个或多个`@Bean`方法，并且可以由 Spring 容器处理以在运行时为这些 bean 生成 bean 定义和服务请求

## Spring `@Configuration`示例

以下应用使用`@Configuration`来配置 Spring 应用。

```java
pom.xml
src
└───src
    ├───main
    │   ├───java
    │   │   └───com
    │   │       └───zetcode
    │   │           │   Application.java
    │   │           └───config
    │   │                   AppConfig.java
    │   │                   H2Configurer.java
    │   └───resources
    │           application.properties
    │           logback.xml
    └───test
        └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>configurationex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/application.properties`

```java
app.name=My application
app.db=H2

```

这里我们有一些应用属性。

`com/zetcode/config/AppConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;

@Configuration
@ComponentScan(basePackages = "com.zetcode")
@PropertySource(value = "application.properties")
public class AppConfig {

    @Bean
    public H2Configurer databaseConfig() {
        return new H2Configurer();
    }
}

```

`AppConfig`是应用配置类。 它装饰有`@Configuration`注释，这是`@Component`的一种特殊形式。

```java
@Configuration
@ComponentScan(basePackages = "com.zetcode")
@PropertySource(value = "application.properties")
public class AppConfig {

```

通过`@ComponentScan`启用组件扫描，并通过`@PropertySource`加载资源。

```java
@Bean
public H2Configurer databaseConfig() {
    return new H2Configurer();
}

```

使用`@Bean`注解，我们创建了`H2Configurer` bean。

`com/zetcode/config/H2Configurer.java`

```java
package com.zetcode.config;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class H2Configurer {

    private static final Logger logger = LoggerFactory.getLogger(H2Configurer.class);

    public H2Configurer() {

        logger.info("Configuring H2 database");
    }
}

```

`H2Configurer`仅记录一条消息。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.stereotype.Component;

@Component
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(AppConfig.class);
        var app = ctx.getBean(Application.class);

        app.run();

        ctx.close();
    }

    @Value("${app.name}")
    private String applicationName;

    @Value("${app.db}")
    private String database;

    private void run() {

        logger.info("Application name: {}", applicationName);
        logger.info("Database: {}", database);

    }
}

```

应用类显示应用属性。 这些属性通过`@Value`注入到属性中。

```java
var ctx = new AnnotationConfigApplicationContext(AppConfig.class);

```

`AppConfig`被加载到应用上下文中。

```java
$ mvn -q exec:java
20:07:39.769 INFO  com.zetcode.config.H2Configurer - Configuring H2 database 
20:07:39.801 INFO  com.zetcode.Application - Application name: My application 
20:07:39.816 INFO  com.zetcode.Application - Database: H2 

```

我们运行该应用。

在本教程中，我们使用`@Configuration`配置了一个 Spring 应用。

您可能也对这些相关教程感兴趣： [Spring `@Bean`教程](/spring/bean/)， [Spring `@PropertySource`教程](/spring/propertysource/)， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Java 教程](/lang/java/)，或列出[所有 Spring 教程](/all/#spring)。