# Spring C 命名空间教程

> 原文： [http://zetcode.com/spring/cnamespace/](http://zetcode.com/spring/cnamespace/)

Spring 构造函数名称空间教程展示了如何在 Spring 应用的基于构造函数的注入中使用 C-命名空间。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring C 命名空间

Spring C-命名空间是 XML 快捷方式，并且替代了`<bean/>`标记的`<constructor-arg/>`子元素。 要启用 C 命名空间功能，我们需要将`xmlns:c="http://www.springframework.org/schema/c"`添加到 XML 文件中。 请注意，此命名空间没有单独的 XSD 文件。 因此，诸如 IntelliJ 之类的 IDE 无法识别它。

## Spring C 命名空间示例

该应用包含两个`User` bean。 一个注入旧的`<constructor-arg/>`，另一个注入新的 C-命名空间属性。

```java
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───bean
│   │                   User.java
│   └───resources
│           logback.xml
│           my-beans.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>cnamespace</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`和`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/my-beans.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans xmlns="http://www.springframework.org/schema/beans"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns:c="http://www.springframework.org/schema/c"
        xsi:schemaLocation="http://www.springframework.org/schema/beans
            http://www.springframework.org/schema/beans/spring-beans.xsd">

    <bean name="user1" class="com.zetcode.bean.User">
        <constructor-arg name="name" value="John Doe"/>
        <constructor-arg name="occupation" value="gardener"/>
    </bean>

    <bean name="user2" class="com.zetcode.bean.User"
            c:name="Peter Smith" c:occupation="teacher"/>

</beans>

```

`my-beans.xml`文件声明了两个 bean：`user1`和`user2`。 `user1`使用`<constructor-arg/>`注入其值，而`user2`使用`c:name`和`c:occupation`属性。

`com/zetcode/bean/User.java`

```java
package com.zetcode.bean;

public class User {

    private String name;
    private String occupation;

    public User(String name, String occupation) {
        this.name = name;
        this.occupation = occupation;
    }

    @Override
    public String toString() {

        final var sb = new StringBuilder("User{");
        sb.append("name='").append(name).append('\'');
        sb.append(", occupation='").append(occupation).append('\'');
        sb.append('}');
        return sb.toString();
    }
}

```

这是由 Spring 容器管理的`User`类。 它必须包含一个构造函数，因为我们在应用中使用了基于构造函数的注入。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.GenericXmlApplicationContext;

public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new GenericXmlApplicationContext("my-beans.xml");

        var u1 = ctx.getBean("user1");
        var u2 = ctx.getBean("user2");

        logger.info("{}", u1);
        logger.info("{}", u2);

        ctx.close();
    }
}

```

这是主要的应用类。 它检索两个 bean 并将它们打印到控制台。

```java
$ mvn -q exec:java
16:40:39.632 INFO  com.zetcode.Application - User{name='John Doe', occupation='gardener'} 
16:40:39.632 INFO  com.zetcode.Application - User{name='Peter Smith', occupation='teacher'} 

```

我们运行该应用。

在本教程中，我们展示了如何在 C-命名空间中使用基于构造函数的注入。

您可能也对这些相关教程感兴趣： [Spring `@Qualifier`注释教程](/spring/qualifier/)， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring P-命名空间教程](/spring/pnamespace/)， [Spring `BeanDefinitionBuilder`教程](/spring/beandefinitionbuilder/)， [Spring HikariCP 教程](/articles/springhikaricp/)和 [Java 教程](/lang/java/)。