# Spring bean 参考教程

> 原文： [http://zetcode.com/spring/beanreference/](http://zetcode.com/spring/beanreference/)

Spring bean 参考教程展示了如何在 Spring 应用的 XML 配置文件中引用 bean。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `ref`属性

`ref`属性是`<ref>`标签的快捷方式，用于引用其他注入的 bean。

## Spring bean 参考示例

该应用包含两个 bean：`infoMessage`和`mesageRenderer`。 `mesageRenderer`通过`ref`属性引用`infoMessage`。

```java
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───bean
│   │           │       IMessage.java
│   │           │       InfoMessage.java
│   │           └───service
│   │                   MessageRenderer.java
│   └───resources
│           logback.xml
│           my-beans.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>beanreference</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`和`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} [%thread] %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/my-beans.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans xmlns="http://www.springframework.org/schema/beans"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xsi:schemaLocation="http://www.springframework.org/schema/beans
            http://www.springframework.org/schema/beans/spring-beans.xsd">

    <bean id="infoMessage" class="com.zetcode.bean.InfoMessage"/>

    <bean name="messageRenderer" class="com.zetcode.service.MessageRenderer">
            <constructor-arg name="message" ref="infoMessage"/>
    </bean>

</beans>

```

`my-beans.xml`文件声明了两个 bean：`info`和`messageRenderer`。 `infoMessage`通过`ref`属性引用`infoMessage`。 Spring 通过构造函数注入将`infoMessage` bean 注入`message`属性。

`com/zetcode/bean/IMessage.java`

```java
package com.zetcode.bean;

public interface IMessage {

    String getMessage();
}

```

`IMessage`接口具有一个方法声明。

`com/zetcode/bean/InfoMessage.java`

```java
package com.zetcode.bean;

public class InfoMessage implements IMessage {

    public String getMessage() {

        return "This is information message";
    }
}

```

`InfoMessage` bean 返回一条信息消息。

`com/zetcode/service/MessageRenderer.java`

```java
package com.zetcode.service;

import com.zetcode.Application;
import com.zetcode.bean.IMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MessageRenderer {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    private IMessage message;

    public MessageRenderer(IMessage message) {
        this.message = message;
    }

    public void renderMessage() {

        logger.info("{}", message.getMessage());
    }
}

```

`MessageRenderer`呈现一条消息。 它期望注入一个 bean。 Spring 通过构造函数注入将`infoMessage` bean 注入`MessageRenderer`。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.service.MessageRenderer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.GenericXmlApplicationContext;

public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new GenericXmlApplicationContext("my-beans.xml");
        var renderer =  (MessageRenderer) ctx.getBean("messageRenderer");

        renderer.renderMessage();

        ctx.close();
    }
}

```

这是主要的应用类。 它检索`MessageRenderer` bean 并调用其`renderMessage()`方法。

```java
$ mvn -q exec:java
12:05:23.567 [com.zetcode.Application.main()] INFO  com.zetcode.Application - This is information message

```

我们运行该应用。

在本教程中，我们展示了如何使用 Spring 的 XML `ref`属性引用其他 bean。

您可能也对这些相关教程感兴趣： [Spring `@Qualifier`注释教程](/spring/qualifier/)， [Spring `context:property-placeholder`教程](/spring/propertyplaceholder/)， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring 注入列表 XML 教程](/spring/injectlistxml/)， [Spring `BeanDefinitionBuilder`教程](/spring/beandefinitionbuilder/)， [Spring HikariCP 教程](/articles/springhikaricp/)和 [Java 教程](/lang/java/)。