# Spring `@Qualifier`注释教程

> 原文： [http://zetcode.com/spring/qualifier/](http://zetcode.com/spring/qualifier/)

Spring `@Qualifier`注释教程显示了如何使用`@Qualifier`来区分 Spring 应用中的 bean。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@Qualifier`注解

如果 Spring 不能这样做，则`@Qualifier`注释有助于消除 Bean 引用的歧义。

## Spring `@Qualifier`示例

该应用具有不同类型的消息 bean。 我们用`@Qualifier`区分它们。

```java
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───bean
│   │           │       IMessage.java
│   │           │       Info.java
│   │           │       Warning.java
│   │           └───service
│   │                   MessageProducer.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>qualifierannotation</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`和`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} [%thread] %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/bean/IMessage.java`

```java
package com.zetcode.bean;

public interface IMessage {

    String getMessage();
}

```

`IMessage`接口具有一个方法声明。

`com/zetcode/bean/Info.java`

```java
package com.zetcode.bean;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

@Component
@Qualifier("info")
public class Info implements IMessage {

    @Override
    public String getMessage() {

        return "This is an information message";
    }
}

```

接口的第一个实现提供了一条信息消息。 `@Qualifier`用于标识 bean。

`com/zetcode/bean/Warning.java`

```java
package com.zetcode.bean;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

@Component
@Qualifier("warning")
public class Warning implements IMessage {

    public String getMessage() {

        return "This is a warning message";
    }
}

```

第二种实现给出警告消息。 它也以`@Qualifier`命名。

`com/zetcode/service/MessageProducer.java`

```java
package com.zetcode.service;

import com.zetcode.bean.IMessage;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

@Service
public class MessageProducer {

    private static final Logger logger = LoggerFactory.getLogger(MessageProducer.class);

    @Autowired
    @Qualifier("info")
    private IMessage infoMessage;

    @Autowired
    @Qualifier("warning")
    private IMessage warningMessage;

    public void produce() {

        logger.info("{}", infoMessage.getMessage());
        logger.warn("{}", warningMessage.getMessage());
    }
}

```

`MessageProducer`注入两个`IMessage` bean。 为了区分它们，我们使用`@Qualifier`注解。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.service.MessageProducer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;

@ComponentScan(basePackages = "com.zetcode.bean;com.zetcode.service")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        logger.info("Application starting");

        try (var ctx = new AnnotationConfigApplicationContext(Application.class)) {

            var messageProducer = (MessageProducer) ctx.getBean("messageProducer");
            messageProducer.produce();
        }
    }
}

```

这是主要的应用类。 它检索`messageProducer` bean 并调用其`produce()`方法。

```java
$ mvn -q exec:java
10:50:03.309 [com.zetcode.Application.main()] INFO  com.zetcode.Application - Application starting
10:50:03.574 [com.zetcode.Application.main()] INFO  com.zetcode.service.MessageProducer - This is an information message
10:50:03.574 [com.zetcode.Application.main()] WARN  com.zetcode.service.MessageProducer - This is a warning message

```

我们运行该应用。

在本教程中，我们使用了 Spring 的`@Qualifier`注解。

您可能也对这些相关教程感兴趣： [Spring `@Bean`教程](/spring/bean/)， [Spring Boot `@Qualifier`注解](/articles/springbootqualifier/)， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。