# Spring `@Autowired`教程

> 原文： [http://zetcode.com/spring/autowired/](http://zetcode.com/spring/autowired/)

Spring `@Autowired`教程展示了如何使用`@Autowired`注解在 Spring 应用中注入依赖项。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@Autowired`

`@Autowired`注解标记将由 Spring 的依赖项注入工具自动装配的构造函数，字段，设置器方法或配置方法。 它是 JSR-330 `@Inject`注释的替代方法。

## Spring `@Autowired`示例

应用使用`@Autowired`注入依赖项。 依赖关系是一个返回单词的服务对象。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───service
│   │                   WordService.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springautowired</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>      

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/service/WordService.java`

```java
package com.zetcode.service;

import org.springframework.stereotype.Service;
import java.util.List;
import java.util.Random;

@Service
public class WordService {

    private final List<String> words = List.of("pen", "sky",
            "rock", "forest", "falcon", "eagle");

    public List<String> all() {

        return words;
    }

    public String randomWord() {

        return words.get(new Random().nextInt(words.size()));
    }
}

```

`WordService`类带有`@Service`注解。 Spring 在组件扫描的帮助下将其注册为托管 Bean。 稍后将此服务对象与`@Autowired`一起注入`Application`中。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.service.WordService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.stereotype.Component;

@Component
@ComponentScan(basePackages="com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    @Autowired
    private WordService wordService;

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);

        var bean = ctx.getBean(Application.class);
        bean.run();

        ctx.close();
    }

    public void run() {

        logger.info("{}", wordService.randomWord());
        logger.info("{}", wordService.randomWord());

        var words = wordService.all();
        words.stream().forEach(word -> logger.info("{}", word));
    }
}

```

使用`WordService`使用输出字注释应用。 服务依赖项与`@Autowired`一起注入`Application`中。

```java
@Autowired
private WordService wordService;

```

这称为场注入。

> **注意**：虽然字段注入简短而有趣，但通常建议使用构造函数注入或设置器注入。

```java
$ mvn -q exec:java
17:15:34.504 INFO  com.zetcode.Application - falcon
17:15:34.507 INFO  com.zetcode.Application - eagle
17:15:34.508 INFO  com.zetcode.Application - pen
17:15:34.508 INFO  com.zetcode.Application - sky
17:15:34.509 INFO  com.zetcode.Application - rock
17:15:34.509 INFO  com.zetcode.Application - forest
17:15:34.510 INFO  com.zetcode.Application - falcon
17:15:34.510 INFO  com.zetcode.Application - eagle

```

我们运行该应用。

在本教程中，我们使用`@Autowired`在 Spring 中注入了依赖项。

您可能也对这些相关教程感兴趣： [Spring `@Bean`注释教程](/spring/bean/)， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。