# Spring 概要文件 XML 教程

> 原文： [http://zetcode.com/spring/profilexml/](http://zetcode.com/spring/profilexml/)

Spring 概要文件 XML 教程显示了如何为 Spring 应用配置 XML 概要文件。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring 简介

概要文件是为特定环境（例如开发或生产）声明的一组命名 Bean。 可以使用 XML 或使用注释来配置配置文件。

## Spring 配置文件示例

该应用定义了两个配置文件：生产和开发。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │               Application.java
│   └───resources
│           database-dev.properties
│           database-prod.properties
│           logback.xml
└───test
    └───java

```

这是项目结构。 我们有两个属性文件：`database-dev.properties`和`database-prod.properties`。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>profileex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>      

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-jdbc</artifactId>
            <version>${spring-version}</version>
        </dependency>        

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`，`spring-context`，`spring-jdbc`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`resources/database-dev.properties`

```java
db.url=jdbc:h2:mem:testdb
db.username=testuser
db.password=s$cret

```

这些是开发属性。

`resources/database-prod.properties`

```java
db.url=jdbc:postgresql://localhost/mydb
db.username=user7
db.password=s$cret

```

这些是生产属性。

`resources/my-beans.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans xmlns="http://www.springframework.org/schema/beans"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns:context="http://www.springframework.org/schema/context"
        xsi:schemaLocation="http://www.springframework.org/schema/beans
            http://www.springframework.org/schema/beans/spring-beans.xsd
            http://www.springframework.org/schema/context
            http://www.springframework.org/schema/context/spring-context.xsd">

    <context:property-placeholder location="classpath:database-${spring.profiles.active}.properties"/>

    <bean id="dataSource" class="org.springframework.jdbc.datasource.SimpleDriverDataSource">
        <property name="url" value="${db.url}"/>
        <property name="username" value="${db.username}"/>
        <property name="password" value="${db.password}"/>
    </bean>

</beans>

```

`my-beans.xml`包含`dataSource` bean。 `<context:property-placeholder>`标签基于活动的 Spring 配置文件为数据源配置属性。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.GenericXmlApplicationContext;
import org.springframework.jdbc.datasource.SimpleDriverDataSource;

public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        System.setProperty("spring.profiles.active", "prod");

        var ctx = new GenericXmlApplicationContext("my-beans.xml");

        var dataSource = (SimpleDriverDataSource) ctx.getBean("dataSource");

        logger.info("Url: {}", dataSource.getUrl());
        logger.info("User name: {}", dataSource.getUsername());
        logger.info("Password: {}", dataSource.getPassword());

        ctx.close();
    }
}

```

该应用检索`dataSource` bean 并打印其属性。 激活属性是通过`System.setProperty()`方法设置的。 另外，我们可以将属性设置为 VM 变量`-Dspring.profiles.active=prod`或在 IDE 设置中。

```java
$ mvn -q -Dspring.profiles.active=dev exec:java
20:30:45.832 INFO  com.zetcode.Application - Url: jdbc:h2:mem:testdb
20:30:45.832 INFO  com.zetcode.Application - User name: testuser
20:30:45.832 INFO  com.zetcode.Application - Password: s$cret

```

我们在命令行上设置了配置文件的情况下运行应用

在本教程中，我们为 Spring 应用创建了开发和生产数据。

您可能也对这些相关教程感兴趣： [Spring `@Bean`注释教程](/spring/bean/)， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring `@ComponentScan`教程](/spring/componentscan/)， [Spring 注入列表 XML 教程](/spring/injectlistxml/)， [Spring `BeanDefinitionBuilder`教程](/spring/beandefinitionbuilder/)， [Spring HikariCP 教程](/articles/springhikaricp/)和 [Java 教程](/lang/java/)。