# Spring `DefaultServlet`教程

> 原文： [http://zetcode.com/spring/defaultservlet/](http://zetcode.com/spring/defaultservlet/)

Spring `DefaultServlet`教程展示了如何在 Spring 应用中启用默认 servlet。

Spring 是流行的 Java 应用框架。 在本教程中，我们使用 Spring 5 版本。

## `DefaultServlet`

`DefaultServlet`是大多数 Web 应用的默认资源服务 Servlet，用于提供 HTML 页面和图像等静态资源。

`DefaultServletHttpRequestHandler`尝试在启动时自动检测容器的默认 Servlet，例如 Tomcat，Jetty，Wildfly 和 Resin。 如果默认 Servlet 是使用其他名称自定义配置的，则必须明确提供默认 Servlet 的名称。

如果我们重写`DefaultServlet`的路由（`/`），则可以使用`DefaultServletHandlerConfigurer`的`enable()`方法启用它，以便我们仍然可以使用容器的默认 Servlet 提供静态资源。

## Spring `DefaultServlet`示例

在以下应用中，我们将 Spring 调度程序 servlet 配置为`/`路径，该路径将重写默认 servlet 的路径。 我们使用`DefaultServletHandlerConfigurer`启用默认 servlet。

该应用提供一个简单的 HTML 主页，这是一个静态资源。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           └───config
│   │                   MyWebInitializer.java
│   │                   WebConfig.java
│   ├───resources
│   │       logback.xml
│   └───webapp
│           index.html
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>defaultservletex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <dependencies>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

        </plugins>
    </build>

</project>

```

这是 Maven 构建文件。 我们具有以下依赖性：用于 Java Servlet 技术的`javax.servlet-api`，用于日志记录的`logback-classic`和用于创建 Spring Web MVC 应用的`spring-webmvc`。

`maven-war-plugin`创建 Web 存档（WAR）。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {
        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {
        return new String[]{"/"};
    }
}

```

`MyWebInitializer`初始化 Spring Web 应用。

```java
@Override
protected String[] getServletMappings() {
    return new String[]{"/"};
}

```

我们将Spring `DispatcherServlet`注册到`/`路径。 这代替了`DefaultServlet`； 因此，我们必须在配置文件中注册一个默认的 servlet 处理器。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.DefaultServletHandlerConfigurer;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
@EnableWebMvc
public class WebConfig implements WebMvcConfigurer {

    @Override
    public void configureDefaultServletHandling(DefaultServletHandlerConfigurer configurer) {
        configurer.enable();
    }
}

```

`WebConfig`通过`@EnableWebMvc`启用 Spring MVC，并通过`DefaultServletHandlerConfigurer`的`enable()`方法配置`DefaultServlet`。

```java
@Override
public void configureDefaultServletHandling(DefaultServletHandlerConfigurer configurer) {
    configurer.enable();
}

```

`configureDefaultServletHandling()`使用 URL 映射`/**`和相对于其他 URL 映射的最低优先级来配置`DefaultServletHttpRequestHandler`。 这样，静态资源请求由容器的默认 Servlet 处理。

`webapp/index.html`

```java
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Home page</title>
</head>
<body>

<p>
    This is home page.
</p>

</body>
</html>

```

这是主页。 它是静态资源，由`DefaultServlet`自动提供服务。

```java
$ curl localhost:8080
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Home page</title>
</head>
<body>

<p>
    This is home page.
</p>

</body>
</html>    

```

当我们运行应用时，将提供主页。

在本教程中，我们展示了如何在 Spring 应用中注册默认 servlet。 您可能也对相关教程感兴趣：[独立的 Spring 应用](/articles/standalonespring/)，[Spring WebSocket 教程](/spring/websocket/)，[Spring 自定义 404 错误页面教程](/spring/custom404page/)，Spring `WebApplicationInitializer`教程，[Spring Web 应用简介](/spring/webapplicationinitializer/)，[Java 教程](/lang/java/)。