# Spring `@PostMapping`教程

> 原文： [http://zetcode.com/spring/postmapping/](http://zetcode.com/spring/postmapping/)

Spring `@PostMapping`教程显示了如何使用`@PostMapping`注解将 HTTP POST 请求映射到特定的处理器方法。

Spring 是用于创建企业应用的流行 Java 应用框架。

## `@PostMapping`

`@PostMapping`注释将 HTTP POST 请求映射到特定的处理器方法。 它是一个组合的注释，用作`@RequestMapping(method = RequestMethod.POST)`的快捷方式。

## Spring `@PostMapping`示例

以下应用使用`@PostMapping`创建新资源。 在此示例中，我们使用注解来设置 Spring Web 应用。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           ├───config
│   │           │       MyWebInitializer.java
│   │           │       WebConfig.java
│   │           ├───controller
│   │           │       MyController.java
│   │           ├───model
│   │           │       Post.java
│   │           └───service
│   │                   PostService.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>postmappingex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

        <dependency>
            <groupId>com.fasterxml.jackson.core</groupId>
            <artifactId>jackson-databind</artifactId>
            <version>2.9.8</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>

        </plugins>
    </build>
</project>

```

在`pom.xml`文件中，我们具有项目依赖项。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

@Configuration
public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {

        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {

        return new String[]{"/"};
    }
}

```

`MyWebInitializer`注册 Spring `DispatcherServlet`，它是 Spring Web 应用的前端控制器。

```java
@Override
protected Class<?>[] getServletConfigClasses() {

    return new Class[]{WebConfig.class};
}

```

`getServletConfigClasses()`返回 Web 配置类。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;

@Configuration
@EnableWebMvc
@ComponentScan(basePackages = {"com.zetcode"})
public class WebConfig {

}

```

`WebConfig`通过`@EnableWebMvc`启用 Spring MVC 注解，并为`com.zetcode`包配置组件扫描。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.model.Post;
import com.zetcode.service.PostService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.util.UriComponents;
import org.springframework.web.util.UriComponentsBuilder;

import javax.servlet.http.HttpServletRequest;
import java.util.Set;

import static org.springframework.http.ResponseEntity.ok;

@Controller
public class MyController {

    @Autowired
    private PostService postService;

    @GetMapping(value="/posts")
    public ResponseEntity<Set<Post>> all() {
        return ok().body(postService.all());
    }

    @PostMapping(value = "/posts")
    public ResponseEntity<Post> createPost(HttpServletRequest request,
                                        UriComponentsBuilder uriComponentsBuilder) {

        var content = request.getParameter("content");

        var post = new Post();
        post.setContent(content);

        post = postService.save(post);

        UriComponents uriComponents =
                uriComponentsBuilder.path("/posts/{id}").buildAndExpand(post.getId());
        var location = uriComponents.toUri();

        return ResponseEntity.created(location).build();
    }
}

```

`MyController`提供请求路径和处理器方法之间的映射。

> **注意**：这是一个很好的做法，在响应头返回新创建资源的位置。

```java
@PostMapping(value = "/posts")
public ResponseEntity<Post> createPost(HttpServletRequest request,
                                    UriComponentsBuilder uriComponentsBuilder) {

```

`@PostMapping`将`createPost()`方法映射到`/posts` URL。

```java
var content = request.getParameter("content");

```

我们获得 POST 请求的`content`参数。

```java
var post = new Post();
post.setContent(content);

post = postService.save(post);

```

创建一个新帖子，并将其保存在一个帖子服务中。

```java
UriComponents uriComponents =
    uriComponentsBuilder.path("/posts/{id}").buildAndExpand(post.getId());
var location = uriComponents.toUri();

```

使用`UriComponentsBuilder`构建位置 URI。

```java
return ResponseEntity.created(location).build();

```

返回带有位置 URI 的响应实体。

`com/zetcode/model/Post.java`

```java
package com.zetcode.model;

import java.util.Objects;

public class Post {

    private Long id;
    private String content;

    public Post() {

    }

    public Post(Long id, String content) {
        this.id = id;
        this.content = content;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Post post = (Post) o;
        return Objects.equals(id, post.id) &&
                Objects.equals(content, post.content);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, content);
    }
}

```

这是一个简单的`Post` bean。 它具有两个属性：`id`和`content`。

`com/zetcode/service/PostService.java`

```java
package com.zetcode.service;

import com.zetcode.model.Post;
import org.springframework.stereotype.Service;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.atomic.AtomicLong;

@Service
public class PostService {

    private final AtomicLong counter = new AtomicLong();

    private final Set<Post> posts = new HashSet<>(Set.of(new Post(counter.incrementAndGet(), "Post one"),
            new Post(counter.incrementAndGet(), "Post two"), new Post(counter.incrementAndGet(), "Post three"),
            new Post(counter.incrementAndGet(), "Post four")));

    public Post save(Post p) {

        var post = new Post(counter.incrementAndGet(), p.getContent());
        this.posts.add(post);

        return post;
    }

    public Set<Post> all() {

        return this.posts;
    }
}

```

`PostService`具有保存帖子并返回所有帖子的方法。 我们没有实现数据库层。 相反，我们使用一个简单的内存集合。

```java
$ mvn jetty:run

```

我们运行 Jetty 服务器。

```java
$ curl -i -d "content=Post five" -X POST http://localhost:8080/posts
HTTP/1.1 201 Created
Date: Tue, 30 Apr 2019 09:49:10 GMT
Location: http://localhost:8080/posts/5
Content-Length: 0
Server: Jetty(9.4.14.v20181114)

```

创建一个新帖子。 注意位置标头。

```java
$ curl localhost:8080/posts
[{"id":3,"content":"Post three"},{"id":4,"content":"Post four"},
{"id":1,"content":"Post one"},{"id":5,"content":"Post five"},{"id":2,"content":"Post two"}]

```

我们得到所有帖子。

在本教程中，我们介绍了`@PostMapping`注解。

您可能也对这些相关教程感兴趣： [Spring `@GetMapping`教程](/spring/getmapping/)， [Spring `@RequestMapping`教程](/spring/requestmapping/)， [Spring `@MatrixVariable`教程](/spring/matrixvariable/)， [Java 教程](/lang/java/)，或列出[所有 Spring 教程](/all/#spring)。