# Spring `@PathVariable`教程

> 原文： [http://zetcode.com/spring/pathvariable/](http://zetcode.com/spring/pathvariable/)

Spring `@PathVariable`教程显示了如何读取带有`@PathVariable`注解的 URL 模板变量。 我们创建一个 Spring RESTful 应用来演示注释。

Spring 是用于创建企业应用的流行 Java 应用框架。

## `@PathVariable`

`@PathVariable`是 Spring 注释，指示方法参数应绑定到 URI 模板变量。 如果方法参数为`Map<String, String>`，则将使用所有路径变量名称和值填充映射。

它具有以下可选元素：

*   `name` - 要绑定到的路径变量的名称
*   `required` - 指示路径变量是否为必需
*   `value` - 名称的别名

## Spring `@PathVariable`示例

以下示例创建一个使用`@PathVariable`的 Spring Web 应用。 记录变量值。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           ├───config
│   │           │       MyWebInitializer.java
│   │           │       WebConfig.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是 Spring 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>pathvariableex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>

        </plugins>
    </build>
</project>

```

我们声明项目依赖项。 `@PathVariable`来自`spring-webmvc`封装。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

@Configuration
public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {

        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {

        return new String[]{"/"};
    }
}

```

`DispatcherServlet`是 Spring Web 应用的前端控制器，已注册在`MyWebInitializer`中。

```java
@Override
protected Class<?>[] getServletConfigClasses() {

    return new Class[]{WebConfig.class};
}

```

`getServletConfigClasses()`返回 Web 配置类。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;

@Configuration
@EnableWebMvc
@ComponentScan(basePackages = {"com.zetcode"})
public class WebConfig {

}

```

`WebConfig`通过`@EnableWebMvc`启用 Spring MVC 注解，并为`com.zetcode`包配置组件扫描。

`com/zetcode/MyController.java`

```java
package com.zetcode.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

@RestController
public class MyController {

    private static final Logger logger = LoggerFactory.getLogger(MyController.class);

    @ResponseStatus(value = HttpStatus.OK)
    @GetMapping(value = "/user/{name}")
    public void process(@PathVariable String name) {

        logger.info("User name: {}", name);
    }

    @ResponseStatus(value = HttpStatus.OK)
    @GetMapping(value = "/user/{name}/{email}")
    public void process2(@PathVariable String name, @PathVariable String email) {

        logger.info("User name: {} and email: {}", name, email);
    }

    @ResponseStatus(value = HttpStatus.OK)
    @GetMapping(value = "/book/{author}/{title}")
    public void process3(@PathVariable Map<String, String> vals) {

        logger.info("{}: {}", vals.get("author"), vals.get("title"));

    }
}

```

我们为 GET 请求提供了三个映射。

```java
@ResponseStatus(value = HttpStatus.OK)
@GetMapping(value = "/user/{name}")
public void process(@PathVariable String name) {

    logger.info("User name: {}", name);
}

```

在此代码中，URI 模板变量绑定到`name`方法参数。

```java
@ResponseStatus(value = HttpStatus.OK)
@GetMapping(value = "/user/{name}/{email}")
public void process2(@PathVariable String name, @PathVariable String email) {

    logger.info("User name: {} and email: {}", name, email);
}

```

通过指定多个`@PathVariable`注解，也可以绑定多个变量。

```java
@ResponseStatus(value = HttpStatus.OK)
@GetMapping(value = "/book/{author}/{title}")
public void process3(@PathVariable Map<String, String> vals) {

    logger.info("{}: {}", vals.get("author"), vals.get("title"));
}

```

也可以使用`Map<String, String>`绑定多个变量。

```java
$ mvn jetty:run

```

我们启动 Jetty 服务器。

```java
$ curl localhost:8080/user/Peter/peter@gmail.com/

```

我们用`curl`发出请求。

```java
22:04:35.273 INFO  com.zetcode.controller.MyController - User name: Peter and email: peter@gmail.com

```

应用记录此消息。

在本教程中，我们使用 Spring 框架创建了一个 RESTful Web 应用。 我们已经演示了`@PathVariable`的用法。 您可能也对相关教程感兴趣： [Spring `@RequestMapping`教程](/spring/requestmapping/)， [Spring `@RequestHeader`教程](/spring/requestheader/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。